#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Auxiliary script used to send data between ports on guests.

:copyright: 2010 Red Hat, Inc.
:author: Jiri Zupka (jzupka@redhat.com)
:author: Lukas Doktor (ldoktor@redhat.com)
"""
from __future__ import division
import threading
from threading import Thread
import os
import select
import re
import random
import sys
import array
import stat
import traceback
import signal
import time
import platform
try:
    import six
except ImportError:
    import subprocess
    subprocess.check_call([sys.executable, "-m", "pip", "install", "six"])
from six.moves import xrange
from six.moves import input
from distutils.version import LooseVersion  # pylint: disable=W0611,E0611


if os.name == "posix":  # Linux
    os_linux = True
    import fcntl
else:   # Windows
    os_linux = False
    try:
        import win32file
    except ImportError as failure_detail:
        print("Import failed. Do you have ctypes and pywin32 installed?")
        raise failure_detail

DEBUGPATH = "/sys/kernel/debug"
SYSFSPATH = "/sys/class/virtio-ports/"
DEVPATH = "/dev/virtio-ports/"

exiting = False
virt = None


class VirtioGuest:

    """
    Test tools of virtio_ports.
    """
    LOOP_NONE = 0
    LOOP_POLL = 1
    LOOP_SELECT = 2
    LOOP_RECONNECT_NONE = 3

    def __init__(self):
        self.files = {}
        self.exit_thread = threading.Event()
        self.threads = []
        self.ports = {}
        self.poll_fds = {}
        self.catch_signal = None
        self.use_config = threading.Event()

    def init(self, in_files):
        """
        Init and check port properties.
        """
        raise NotImplementedError

    def _open(self, in_files):
        """
        Open devices and return array of descriptors

        :param in_files: Files array
        :return: Array of descriptor
        """
        raise NotImplementedError

    def poll(self, port, expected, timeout=500):
        """
        Checks the port POLL status and verify with expected results.

        :param port: Port name.
        :param expected: Expected POLL status (mask)
        """
        raise NotImplementedError

    def lseek(self, port, pos, how):
        """
        Use lseek on the device. The device is unseekable so PASS is returned
        when lseek command fails and vice versa.

        :param port: Name of the port
        :param pos: Offset
        :param how: Relative offset os.SEEK_{SET,CUR,END}
        """
        raise NotImplementedError

    def blocking(self, port, mode=False):
        """
        Set port function mode blocking/nonblocking

        :param port: port to set mode
        :param mode: False to set nonblock mode, True for block mode
        """
        raise NotImplementedError

    def asynchronous(self, port, mode=True, exp_val=0):
        """
        Set port function mode async/sync.

        :param port: port which should be pooled.
        :param mode: False to set sync mode, True for sync mode.
        :param exp_val: Value which should be pooled.
        """
        raise NotImplementedError

    def close(self, port_file):
        """
        Close open port.

        :param port_file: File to close.
        """
        raise NotImplementedError

    def open(self, in_file):
        """
        Direct open devices.

        :param in_file: Array of files.
        :return: Array of descriptors.
        """
        raise NotImplementedError

    def loopback(self, in_files, out_files, cachesize=1024,
                 mode=0):
        """
        Start a switch thread.

        (There is a problem with multiple opens of a single file).

        :param in_files: Array of input files.
        :param out_files: Array of output files.
        :param cachesize: Cachesize.
        :param mode: Mode of switch.
        """
        in_f = self._open(in_files)
        out_f = self._open(out_files)

        s = self.Switch(in_f, out_f, self.exit_thread, cachesize, mode)
        s.start()
        self.threads.append(s)
        print("PASS: Start switch")

    def exit_threads(self):
        """
        Function end all running data switch.
        """
        raise NotImplementedError

    def send_loop_init(self, port, length):
        """
        Prepares the sender thread. Requires clean thread structure.

        :param port: On which port to sent data
        :param length: length of data
        """
        raise NotImplementedError

    def send_loop(self):
        """
        Start sender data transfer. Requires senderprepare run first.
        """
        raise NotImplementedError

    def send(self, port, length=1, mode=True, is_static=False):
        """
        Send a data of arbitrary length

        :param port: Port to write data
        :param length: Length of data
        :param mode: True = loop mode, False = one shoot mode
        :param is_static: False = generates $length long block (mode=0)
                          True = generates 4096 long block (faster, mode=1)
        """
        raise NotImplementedError

    def recv(self, port, length=1, bfr=1024, mode=True):
        """
        Receive a data of arbitrary length.

        :param port: Port to write data
        :param length: Length of data
        :param mode: True = loop mode, False = one shoot mode
        """
        raise NotImplementedError

    def clean_port(self, port, bfr=1024):
        raise NotImplementedError


class VirtioGuestPosix(VirtioGuest):

    """
    Test tools of virtio_ports.
    """

    def _readfile(self, name):
        """
        Read file and return content as string

        :param name: Name of file
        :return: Content of file as string
        """
        out = ""
        try:
            f = open(name, "r")
            out = f.read()
            f.close()
        except Exception:
            print("FAIL: Cannot open file %s" % (name))

        return out

    def _get_port_status(self, in_files=None):
        """
        Get info about ports from kernel debugfs.

        :param in_files: Array of input files.
        :return: Ports dictionary of port properties
        """
        ports = {}
        current_version = LooseVersion(platform.uname()[2])
        fixed_version = LooseVersion("4.18.0-97")
        not_present_msg = "FAIL: There's no virtio-ports dir in debugfs"
        if not os.path.ismount(DEBUGPATH):
            os.system('mount -t debugfs none %s' % (DEBUGPATH))
        try:
            if not os.path.isdir('%s/virtio-ports' % (DEBUGPATH)):
                print(not_present_msg)
        except Exception:
            print(not_present_msg)
        else:
            viop_names = os.listdir('%s/virtio-ports' % (DEBUGPATH))
            if in_files is not None:
                dev_names = os.listdir('/dev')
                rep = re.compile(r"vport[0-9]+p[0-9]+")
                dev_names = list(
                    filter(lambda x: rep.match(x) is not None, dev_names))
                if len(dev_names) != len(in_files):
                    print("FAIL: Not all ports were successfully initialized "
                          "in /dev, only %d from %d." % (len(dev_names),
                                                         len(in_files)))
                    return

                if len(viop_names) != len(in_files):
                    print("FAIL: Not all ports were successfully initialized "
                          "in debugfs, only %d from %d." % (len(viop_names),
                                                            len(in_files)))
                    return

            for name in viop_names:
                open_db_file = "%s/virtio-ports/%s" % (DEBUGPATH, name)
                f = open(open_db_file, 'r')
                port = {}
                line_list = []
                for line in iter(f):
                    line_list.append(line)
                try:
                    for line in line_list:
                        m = re.match("(\S+): (\S+)", line)
                        port[m.group(1)] = m.group(2)

                    if port['is_console'] == "yes":
                        port_num = int(port["console_vtermno"])
                        if current_version >= fixed_version:
                            port_num -= 1
                        port["path"] = "/dev/hvc%d" % port_num
                        # Console works like a serialport
                    else:
                        port["path"] = "/dev/%s" % name

                    if not os.path.exists(port['path']):
                        print("FAIL: %s not exist" % port['path'])

                    sysfspath = SYSFSPATH + name
                    if not os.path.isdir(sysfspath):
                        print("FAIL: %s not exist" % (sysfspath))

                    info_name = sysfspath + "/name"
                    port_name = self._readfile(info_name).strip()
                    if port_name != port["name"]:
                        print("FAIL: Port info does not match "
                              "\n%s - %s\n%s - %s" %
                              (info_name, port_name,
                               "%s/virtio-ports/%s" % (DEBUGPATH, name),
                               port["name"]))
                    dev_ppath = DEVPATH + port_name
                    if not os.path.exists(dev_ppath):
                        print("FAIL: Symlink %s does not exist." % dev_ppath)
                    if not os.path.realpath(dev_ppath) != "/dev/name":
                        print("FAIL: Symlink %s is not correct." % dev_ppath)
                except AttributeError:
                    print("Bad data on file %s:\n%s. " %
                          (open_db_file, "".join(line_list).strip()))
                    print("FAIL: Bad data on file %s." % open_db_file)
                    return

                ports[port['name']] = port
                f.close()

        return ports

    def init(self, in_files):
        """
        Init and check port properties.
        """
        self.ports = self._get_port_status(in_files)

        if self.ports is None:
            return
        for item in in_files:
            if (item[1] != self.ports[item[0]]["is_console"]):
                print(self.ports)
                print("FAIL: Host console is not like console on guest side\n")
                return

        print("PASS: Init and check virtioconsole files in system.")

    class Switch(Thread):

        """
        Thread that sends data between ports.
        """

        def __init__(self, in_files, out_files, event,
                     cachesize=1024, method=0):
            """
            :param in_files: Array of input files.
            :param out_files: Array of output files.
            :param method: Method of read/write access.
            :param cachesize: Block to receive and send.
            """
            Thread.__init__(self, name="Switch")
            self.in_files = in_files[0]
            self.in_names = in_files[1]
            self.out_files = out_files[0]
            self.out_names = out_files[1]
            self.exit_thread = event
            self.method = method

            self.cachesize = cachesize

        def _none_mode(self):
            """
            Read and write to device in blocking mode
            """
            while not self.exit_thread.isSet():
                data = b""
                for desc in self.in_files:
                    data += os.read(desc, self.cachesize)
                if data != b"":
                    for desc in self.out_files:
                        os.write(desc, data)

        def _poll_mode(self):
            """
            Read and write to device in polling mode.
            """

            pi = select.poll()
            po = select.poll()

            for fd in self.in_files:
                pi.register(fd, select.POLLIN)

            for fd in self.out_files:
                po.register(fd, select.POLLOUT)

            while not self.exit_thread.isSet():
                data = b""
                t_out = self.out_files

                readyf = pi.poll(1.0)
                for i in readyf:
                    data += os.read(i[0], self.cachesize)

                if data != b"":
                    while ((len(t_out) != len(readyf)) and not
                           self.exit_thread.isSet()):
                        readyf = po.poll(1.0)
                    for desc in t_out:
                        os.write(desc, data)

        def _select_mode(self):
            """
            Read and write to device in selecting mode.
            """
            while not self.exit_thread.isSet():
                ret = select.select(self.in_files, [], [], 1.0)
                data = b""
                if ret[0] != []:
                    for desc in ret[0]:
                        data += os.read(desc, self.cachesize)
                if data != b"":
                    ret = select.select([], self.out_files, [], 1.0)
                    while ((len(self.out_files) != len(ret[1])) and not
                           self.exit_thread.isSet()):
                        ret = select.select([], self.out_files, [], 1.0)
                    for desc in ret[1]:
                        os.write(desc, data)

        def _reconnect_none_mode(self):
            """
            Read and write to device in blocking mode, close and reopen device
            when it get OSError.
            This is workaround for hotplugging of virtio port.
            """
            # TODO: Remove port unplugging after failure from guest_worker
            #       when bz796048 is resolved.
            while not self.exit_thread.isSet():
                data = b""
                for i in xrange(len(self.in_files)):
                    if self.exit_thread.isSet():
                        break
                    desc = self.in_files[i]
                    try:
                        data += os.read(desc, self.cachesize)
                    except OSError as inst:
                        if inst.errno == 9:
                            # Wait 0.1 before spoiling output with additional
                            # log information.
                            # time.sleep(0.5)
                            sys.stdout.write("FD closed, readerr %s\n" % inst)
                            while self.in_names[i] not in virt.files:
                                pass
                            self.in_files[i] = virt.files[self.in_names[i]]
                        else:
                            sys.stdout.write("Missing device, readerr %s\n"
                                             % inst)
                            _desc = desc
                            for item in six.iteritems(virt.files):
                                if item[1] == desc:
                                    path = item[0]
                                    break
                            for item in six.iteritems(virt.ports):
                                if item[1]['path'] == path:
                                    name = item[0]
                                    break
                            virt.close(name)
                            while not self.exit_thread.isSet():
                                try:
                                    desc = virt._open([name])[0]
                                    sys.stdout.write("PASS: Opened %s\n"
                                                     % name)
                                    break
                                except OSError:
                                    pass
                            self.in_files[self.in_files.index(_desc)] = desc
                if data != b"":
                    for i in xrange(len(self.out_files)):
                        if self.exit_thread.isSet():
                            break
                        desc = self.out_files[i]
                        written = False
                        while not written:
                            try:
                                if self.exit_thread.isSet():
                                    break
                                os.write(desc, data)
                                written = True
                            except OSError as inst:
                                if inst.errno == 9:
                                    # Wait 0.1 before spoiling output with
                                    # additional log information.
                                    # time.sleep(0.5)
                                    sys.stdout.write("FD closed, writeerr %s\n"
                                                     % inst)
                                    while self.out_names[i] not in virt.files:
                                        pass
                                    self.out_files[i] = virt.files[
                                        self.out_names[i]]
                                else:
                                    sys.stdout.write("Missing device, writeerr"
                                                     " %s\n" % inst)
                                    _desc = desc
                                    for item in six.iteritems(virt.files):
                                        if item[1] == desc:
                                            path = item[0]
                                            break
                                    for item in six.iteritems(virt.ports):
                                        if item[1]['path'] == path:
                                            name = item[0]
                                            break
                                    virt.close(name)
                                    while not self.exit_thread.isSet():
                                        try:
                                            desc = virt._open([name])[0]
                                            sys.stdout.write("PASS: Opened "
                                                             "%s\n" % name)
                                            break
                                        except OSError:
                                            pass
                                    _desc = self.out_files.index(_desc)
                                    self.out_files[_desc] = desc

        def run(self):
            if (self.method == VirtioGuest.LOOP_POLL):
                self._poll_mode()
            elif (self.method == VirtioGuest.LOOP_SELECT):
                self._select_mode()
            elif (self.method == VirtioGuest.LOOP_RECONNECT_NONE):
                self._reconnect_none_mode()
            elif (self.method == VirtioGuest.LOOP_NONE):
                self._none_mode()
            else:
                print("WARNIGN: Unknown mode %s, using LOOP_NONE" % self.method)
                self._reconnect_none_mode()

    class Sender(Thread):

        """
        Creates a thread which sends random blocks of data to dst port.
        """

        def __init__(self, port, event, length):
            """
            :param port: Destination port
            :param length: Length of the random data block
            """
            Thread.__init__(self, name="Sender")
            self.port = port
            self.exit_thread = event
            self.data = array.array('L')
            for _ in range(max(length // self.data.itemsize, 1)):
                self.data.append(random.randrange(sys.maxsize))

        def run(self):
            while not self.exit_thread.isSet():
                os.write(self.port, self.data)

    def _open(self, in_files):
        """
        Open devices and return array of descriptors

        :param in_files: Files array
        :return: Array of descriptor
        """
        f = []

        for item in in_files:
            path = self.ports[item]["path"]
            if (path in self.files):
                f.append(self.files[path])
            else:
                try:
                    self.files[path] = os.open(path, os.O_RDWR)
                    if (self.ports[item]["is_console"] == "yes"):
                        print(os.system("stty -F %s raw -echo" % (path)))
                        print(os.system("stty -F %s -a" % (path)))
                    f.append(self.files[path])
                except Exception as inst:
                    print("FAIL: Failed to open file %s" % (path))
                    raise inst
        return f

    @staticmethod
    def pollmask_to_str(mask):
        """
        Conver pool mast to string

        :param mask: poll return mask
        """
        out = ""
        if (mask & select.POLLIN):
            out += "IN "
        if (mask & select.POLLPRI):
            out += "PRI IN "
        if (mask & select.POLLOUT):
            out += "OUT "
        if (mask & select.POLLERR):
            out += "ERR "
        if (mask & select.POLLHUP):
            out += "HUP "
        if (mask & select.POLLMSG):
            out += "MSG "
        return out

    def poll(self, port, expected, timeout=500):
        """
        Pool event from device and print event like text.

        :param file: Device.
        """
        in_f = self._open([port])

        p = select.poll()
        p.register(in_f[0])

        mask = p.poll(timeout)

        maskstr = self.pollmask_to_str(mask[0][1])
        if (mask[0][1] & expected) == expected:
            print("PASS: Events: " + maskstr)
        else:
            emaskstr = self.pollmask_to_str(expected)
            print("FAIL: Events: " + maskstr + "  Expected: " + emaskstr)

    def lseek(self, port, pos, how):
        """
        Use lseek on the device. The device is unseekable so PASS is returned
        when lseek command fails and vice versa.

        :param port: Name of the port
        :param pos: Offset
        :param how: Relative offset os.SEEK_{SET,CUR,END}
        """
        fd = self._open([port])[0]

        try:
            os.lseek(fd, pos, how)
        except Exception as inst:
            if inst.errno == 29:
                print("PASS: the lseek failed as expected")
            else:
                print(inst)
                print("FAIL: unknown error")
        else:
            print("FAIL: the lseek unexpectedly passed")

    def blocking(self, port, mode=False):
        """
        Set port function mode blocking/nonblocking

        :param port: port to set mode
        :param mode: False to set nonblock mode, True for block mode
        """
        fd = self._open([port])[0]

        try:
            fl = fcntl.fcntl(fd, fcntl.F_GETFL)
            if not mode:
                fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)
            else:
                fcntl.fcntl(fd, fcntl.F_SETFL, fl & ~os.O_NONBLOCK)

        except Exception as inst:
            print("FAIL: Setting (non)blocking mode: " + str(inst))
            return

        if mode:
            print("PASS: set to blocking mode")
        else:
            print("PASS: set to nonblocking mode")

    def __call__(self, sig, frame):
        """
        Call function. Used for signal handle.
        """
        if (sig == signal.SIGIO):
            self.sigio_handler(sig, frame)

    def sigio_handler(self, sig, frame):
        """
        Handler for sigio operation.

        :param sig: signal which call handler.
        :param frame: frame of caller
        """
        if self.poll_fds:
            p = select.poll()
            for fd_name in self.poll_fds.keys():
                p.register(fd_name)

            masks = p.poll(1)
            print(masks)
            for mask in masks:
                self.poll_fds[mask[0]][1] |= mask[1]

    def get_sigio_poll_return(self, port):
        """
        Return PASS, FAIL and poll walue in string format.

        :param port: Port to check poll information.
        """
        fd = self._open([port])[0]
        state = self.poll_fds[fd][:]
        print(state)
        end = time.time() + 5   # wait up to 5s for new signals
        while state[0] != state[1]:
            while time.time() < end:
                if state != self.poll_fds[fd]:  # new signal arrived
                    break
                time.sleep(0)   # schedule another thread
            else:
                print("FAIL: Timeout reached, Events: %s  Expected: %s"
                      % (self.pollmask_to_str(state[1]),
                         self.pollmask_to_str(state[0])))
                self.poll_fds[fd][1] = 0
                return
            state = self.poll_fds[fd][:]
            print(state)
        print("PASS: Events: " + self.pollmask_to_str(state[1]))
        self.poll_fds[fd][1] = 0

    def set_pool_want_return(self, port, poll_value):
        """
        Set value to static variable.

        :param port: Port which should be set excepted mask
        :param poll_value: Value to check sigio signal.
        """
        fd = self._open([port])[0]
        self.poll_fds[fd] = [poll_value, 0]
        print("PASS: Events: " + self.pollmask_to_str(poll_value))

    def catching_signal(self):
        """
        return: True if should set catch signal, False if ignore signal and
                none when configuration is not changed.
        """
        ret = self.catch_signal
        self.catch_signal = None
        return ret

    def asynchronous(self, port, mode=True, exp_val=0):
        """
        Set port function mode async/sync.

        :param port: port which should be pooled.
        :param mode: False to set sync mode, True for sync mode.
        :param exp_val: Value which should be pooled.
        """
        fd = self._open([port])[0]

        try:
            fcntl.fcntl(fd, fcntl.F_SETOWN, os.getpid())
            fl = fcntl.fcntl(fd, fcntl.F_GETFL)

            self.use_config.clear()
            if mode:
                self.catch_signal = True
                os.kill(os.getpid(), signal.SIGUSR1)
                self.use_config.wait()
                fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_ASYNC)
                self.poll_fds[fd] = [exp_val, 0]
            else:
                del self.poll_fds[fd]
                fcntl.fcntl(fd, fcntl.F_SETFL, fl & ~os.O_ASYNC)
                self.catch_signal = False
                os.kill(os.getpid(), signal.SIGUSR1)
                self.use_config.wait()

        except Exception as inst:
            print("FAIL: Setting (a)sync mode: " + str(inst))
            return

        if mode:
            print("PASS: Set to async mode")
        else:
            print("PASS: Set to sync mode")

    def close(self, filepath):
        """
        Close open port.

        :param filepath: File to close.
        """
        descriptor = None
        path = self.ports[filepath]["path"]
        if path is not None:
            if path in list(self.files.keys()):
                descriptor = self.files[path]
                del self.files[path]
            if descriptor is not None:
                try:
                    os.close(descriptor)
                except Exception as inst:
                    print("FAIL: Closing the file: " + str(inst))
                    return
        print("PASS: Close")

    def open(self, in_file, attempts=1):
        """
        Direct open devices.

        :param in_file: Array of files.
        :return: Array of descriptors.
        """
        opened = False
        for i in xrange(attempts):
            try:
                name = self.ports[in_file]["path"]
                self.files[name] = os.open(name, os.O_RDWR)
                if (self.ports[in_file]["is_console"] == "yes"):
                    print(os.system("stty -F %s raw -echo" % (name)))
                opened = True
                break
            except Exception as exc:
                print(str(exc))
                time.sleep(0.1)
        if opened:
            print("PASS: All files opened correctly. (%d)" % i)
        else:
            print("FAIL: Failed open file %s" % name)

    def loopback(self, in_files, out_files, cachesize=1024,
                 mode=0):
        """
        Start a switch thread.

        (There is a problem with multiple opens of a single file).

        :param in_files: Array of input files.
        :param out_files: Array of output files.
        :param cachesize: Cachesize.
        :param mode: Mode of switch.
        """
        self.ports = self._get_port_status()

        in_f = self._open(in_files)
        out_f = self._open(out_files)
        in_files = [self.ports[item]["path"] for item in in_files]
        out_files = [self.ports[item]["path"] for item in out_files]

        s = self.Switch([in_f, in_files], [out_f, out_files], self.exit_thread,
                        cachesize, mode)
        s.start()
        self.threads.append(s)
        sys.stdout.write("PASS: Start switch\n")

    def exit_threads(self):
        """
        Function end all running data switch.
        """
        self.exit_thread.set()
        for th in self.threads:
            print("join %s" % th.getName())
            th.join()
        self.exit_thread.clear()

        del self.threads[:]
        for desc in six.itervalues(self.files):
            os.close(desc)
        self.files.clear()
        print("PASS: All threads finished")

    def die(self):
        """
        Quit consoleswitch.
        """
        self.exit_threads()
        sys.exit(0)

    def send_loop_init(self, port, length):
        """
        Prepares the sender thread. Requires clean thread structure.
        """
        self.ports = self._get_port_status()
        in_f = self._open([port])

        self.threads.append(self.Sender(in_f[0], self.exit_thread, length))
        print("PASS: Sender prepare")

    def send_loop(self):
        """
        Start sender data transfer. Requires senderprepare run first.
        """
        self.threads[0].start()
        print("PASS: Sender start")

    def send(self, port, length=1, mode=True, is_static=False):
        """
        Send a data of some length

        :param port: Port to write data
        :param length: Length of data
        :param mode: True = loop mode, False = one shoot mode
        :param is_static: False = generates $length long block (mode=0)
                          True = generates 4096 long block (faster, mode=1)
        """
        in_f = self._open([port])

        data = b""
        writes = 0

        if not is_static:
            while len(data) < length:
                data += b"%c" % random.randrange(255)
            try:
                writes = os.write(in_f[0], data)
            except Exception as inst:
                print(inst)
        else:
            while len(data) < 4096:
                data += b"%c" % random.randrange(255)
        if mode:
            while (writes < length):
                try:
                    writes += os.write(in_f[0], data)
                except Exception as inst:
                    print(inst)
        if writes >= length:
            print("PASS: Send data length %d" % writes)
        else:
            print("FAIL: Partial send: desired %d, transferred %d" %
                  (length, writes))

    def recv(self, port, length=1, bfr=1024, mode=True):
        """
        Receive a data of arbitrary length.

        :param port: Port to write data
        :param length: Length of data
        :param mode: True = loop mode, False = one shoot mode
        """
        in_f = self._open([port])

        recvs = b""
        try:
            recvs = os.read(in_f[0], bfr)
        except Exception as inst:
            print(inst)
        if mode:
            while (len(recvs) < length):
                try:
                    recvs += os.read(in_f[0], bfr)
                except Exception as inst:
                    print(inst)
        if len(recvs) >= length:
            print("PASS: Recv data length %d" % len(recvs))
        else:
            print("FAIL: Partial recv: desired %d, transferred %d" %
                  (length, len(recvs)))

    def clean_port(self, port, bfr=1024):
        in_f = self._open([port])
        ret = select.select([in_f[0]], [], [], 1.0)
        buf = b""
        if ret[0]:
            buf = os.read(in_f[0], bfr)
        print("PASS: Rest in socket: " + str(buf[:10]))


class VirtioGuestNt(VirtioGuest):

    """
    Test tools of virtio_ports.
    """
    LOOP_NONE = 0
    LOOP_POLL = 0   # TODO: Use SELECT instead of NONE (poll not supp. by win)
    LOOP_SELECT = 0     # TODO: Support for Select

    def _get_port_status(self, in_files=[]):
        """
        Get info about ports.

        :param in_files: Array of input files.
        :return: Ports dictionary of port properties
        """
        ports = {}
        for in_file in in_files:
            port = {}
            port['path'] = "\\\\.\\%s" % in_file[0]
            port['name'] = in_file[0]
            port['is_console'] = in_file[1]
            ports[in_file[0]] = port

        return ports

    def init(self, in_files):
        """
        Init and check port properties.
        """
        # This only sets the ports names and paths
        # TODO: symlinks are sometimes missing, use /dev/vport%dp%d"
        self.ports = self._get_port_status(in_files)

        # Check if all ports really exists
        remove = []
        for item in six.iteritems(self.ports):
            port = item[1]
            try:
                hFile = win32file.CreateFile(port['path'], 0, 0, None,
                                             win32file.OPEN_EXISTING,
                                             win32file.FILE_ATTRIBUTE_NORMAL,
                                             None)
                win32file.CloseHandle(hFile)
            except win32file.error:
                remove.append(port['name'])
                print("Fail to open port %s" % port['name'])
        for name in remove:
            del(self.ports[name])

        # Check if in_files count and system port count matches
        # TODO: Not all devices are listed
        # TODO: Find the way to list all devices
        if remove:
            print("FAIL: Not all ports are present, check the log.")
            return
        """
        reg = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, "System")
        reg = _winreg.OpenKey(reg, "CurrentControlSet")
        reg = _winreg.OpenKey(reg, "Services")
        reg = _winreg.OpenKey(reg, "VirtioSerial")
        reg = _winreg.OpenKey(reg, "Enum")
        virtio_port_count = _winreg.QueryValueEx(reg, "Count")[0]
        if virtio_port_count != len(self.ports):
            print("FAIL: Number of ports (%d) doesn't match the number"
                  " of ports in registry (%d)"
                  % (len(self.ports), virtio_port_count))
            return
        """

        print("PASS: Init and check virtioconsole files in system.")

    def close(self, filepath):
        """
        Close open port.

        :param filepath: File to close.
        """
        hFile = None
        path = self.ports[filepath]["path"]
        if path is not None:
            if path in list(self.files.keys()):
                hFile = self.files[path]
                del self.files[path]
            if hFile is not None:
                try:
                    win32file.CloseHandle(hFile)
                except win32file.error as inst:
                    print("FAIL: Closing the file: " + str(inst))
                    return
        print("PASS: Close")

    def _open(self, in_files):
        """
        Open devices and return array of descriptors

        :param in_files: List of port names
        :return: Array of descriptor
        """
        f = []

        for name in in_files:
            path = self.ports[name]["path"]
            if path in self.files:
                f.append(self.files[path])
            else:
                ret = self.open(name)
                if ret:
                    raise ret
                f.append(self.files[path])
        return f

    def open(self, name):
        """
        Direct open devices.

        :param name: Port name.
        :return: 0 on success
        """
        path = self.ports[name]['path']
        try:
            self.files[path] = win32file.CreateFile(path,
                                                    win32file.GENERIC_WRITE |
                                                    win32file.GENERIC_READ,
                                                    0,
                                                    None,
                                                    win32file.OPEN_EXISTING,
                                                    win32file.FILE_ATTRIBUTE_NORMAL,
                                                    None)
        except win32file.error as exc_detail:
            print("%s\nFAIL: Failed open file %s" % (str(exc_detail), name))
            return exc_detail
        print("PASS: All files opened correctly.")

    def exit_threads(self):
        """
        Function end all running data switch.
        """
        self.exit_thread.set()
        for th in self.threads:
            print("join")
            th.join()
        self.exit_thread.clear()

        del self.threads[:]
        for desc in six.itervalues(self.files):
            win32file.CloseHandle(desc)
        self.files.clear()
        print("PASS: All threads finished")

    class Switch(Thread):

        """
        Thread that sends data between ports.
        """

        def __init__(self, in_files, out_files, event,
                     cachesize=1024, method=0):
            """
            :param in_files: Array of input files.
            :param out_files: Array of output files.
            :param method: Method of read/write access.
            :param cachesize: Block to receive and send.
            """
            Thread.__init__(self, name="Switch")

            self.in_files = in_files
            self.out_files = out_files
            self.exit_thread = event
            self.method = method
            self.cachesize = cachesize

        def _none_mode(self):
            """
            Read and write to device in blocking mode
            """
            data = ""
            while not self.exit_thread.isSet():
                data = ""
                for desc in self.in_files:
                    ret, _data = win32file.ReadFile(desc, self.cachesize)
                    if ret:
                        msg = ("Error occurred while receiving data, "
                               "err=%s, read=%s" % (ret, _data))
                        print("FAIL: " + msg)
                        raise IOError(msg)
                    data += _data
                if data != "":
                    for desc in self.out_files:
                        ret, _data = win32file.WriteFile(desc, data)
                        if ret:
                            msg = ("Error occurred while sending data, "
                                   "err=%s, sentlen=%s" % (ret, _data))
                            print("FAIL: " + msg)
                            raise IOError(msg)

        def run(self):
            self._none_mode()

    class Sender(Thread):

        """
        Creates a thread which sends random blocks of data to dst port.
        """

        def __init__(self, port, event, length):
            """
            :param port: Destination port
            :param length: Length of the random data block
            """
            Thread.__init__(self, name="Sender")
            self.port = port
            self.exit_thread = event
            self.data = array.array('L')
            for _ in range(max(length // self.data.itemsize, 1)):
                self.data.append(random.randrange(sys.maxint))

        def run(self):
            while not self.exit_thread.isSet():
                if win32file.WriteFile(self.port, self.data)[0]:
                    msg = "Error occurred while sending data."
                    print("FAIL: " + msg)
                    raise IOError(msg)

    def send_loop_init(self, port, length):
        """
        Prepares the sender thread. Requires clean thread structure.
        """
        in_f = self._open([port])

        self.threads.append(self.Sender(in_f[0], self.exit_thread, length))
        print("PASS: Sender prepare")

    def send_loop(self):
        """
        Start sender data transfer. Requires senderprepare run first.
        """
        self.threads[0].start()
        print("PASS: Sender start")

    def send(self, port, length=1, mode=True, is_static=False):
        """
        Send a data of arbitrary length

        :param port: Port to write data
        :param length: Length of data
        :param mode: True = loop mode, False = one shoot mode
        :param is_static: False = generates $length long block (mode=0)
                          True = generates 4096 long block (faster, mode=1)
        """
        port = self._open([port])[0]

        data = ""
        writes = 0

        if not is_static:
            try:
                while len(data) < length:
                    data += "%c" % random.randrange(255)
                _ret, _len = win32file.WriteFile(port, data)
                if _ret:
                    msg = ("Error occurred while sending data, "
                           "err=%s, sentlen=%s" % (_ret, _len))
                    raise IOError(msg)
                writes = _len
            except Exception as inst:
                print(inst)
        else:
            while len(data) < 4096:
                data += "%c" % random.randrange(255)
        if mode:
            try:
                while (writes < length):
                    _ret, _len = win32file.WriteFile(port, data)
                    if _ret:
                        msg = ("Error occurred while sending data, err=%s"
                               ", sentlen=%s, allsentlen=%s" % (_ret, _len,
                                                                writes))
                        raise IOError(msg)
                    writes += _len
            except Exception as inst:
                print(inst)
        if writes >= length:
            print("PASS: Send data length %d" % writes)
        else:
            print("FAIL: Partial send: desired %d, transferred %d" %
                  (length, writes))

    def recv(self, port, length=1, buflen=1024, mode=True):
        """
        Receive a data of arbitrary length.

        :param port: Port to write data
        :param length: Length of data
        :param mode: True = loop mode, False = one shoot mode
        """
        port = self._open([port])[0]

        recvs = ""
        try:
            _ret, _data = win32file.ReadFile(port, buflen)
            if _ret:
                msg = ("Error occurred while receiving data, "
                       "err=%s, read=%s" % (_ret, _data))
                raise IOError(msg)
            recvs = _data
        except Exception as inst:
            print(inst)
        if mode:
            while (len(recvs) < length):
                try:
                    _ret, _data = win32file.ReadFile(port, buflen)
                    if _ret:
                        msg = ("Error occurred while receiving data, "
                               "err=%s, read=%s, allread=%s" % (_ret, _data,
                                                                len(recvs)))
                        raise IOError(msg)
                except Exception as inst:
                    print(inst)
        if len(recvs) >= length:
            print("PASS: Recv data length %d" % len(recvs))
        else:
            print("FAIL: Partial recv: desired %d, transferred %d" %
                  (length, len(recvs)))


def is_alive():
    """
    Check is only main thread is alive and if guest react.
    """
    if ((os_linux and (threading.activeCount() == 2)) or
            ((not os_linux) and (threading.activeCount() == 1))):
        print("PASS: Guest is ok no thread alive")
    else:
        threads = ""
        for thread in threading.enumerate():
            threads += thread.name + ", "
        print("FAIL: On guest run thread. Active thread:" + threads)


def guest_exit():
    """
    quit/finish/exit this script
    """
    global exiting
    exiting = True


def compile_script():
    """
    Compile virtio_console_guest.py to speed up.
    """
    import py_compile
    py_compile.compile(__file__, "%so" % __file__)
    print("PASS: compile")
    sys.exit(0)


def worker(virt):
    """
    Worker thread (infinite) loop of virtio_guest.
    """
    global exiting
    print("PASS: Daemon start.")
    p = select.poll()
    p.register(sys.stdin.fileno())
    while not exiting:
        d = p.poll()
        if (d[0][1] == select.POLLIN):
            out = input()
            try:
                exec(out)
            except Exception:
                exc_type, exc_value, exc_traceback = sys.exc_info()
                print("On Guest exception from: \n" + "".join(
                    traceback.format_exception(exc_type,
                                               exc_value,
                                               exc_traceback)))
                print("FAIL: Guest command exception.")
        elif (d[0][1] & select.POLLHUP):
            time.sleep(0.5)


def sigusr_handler(sig, frame):
    pass


class Daemon:

    """
    Daemonize guest
    """

    def __init__(self, stdin, stdout, stderr):
        """
        Init daemon.

        :param stdin: path to stdin file.
        :param stdout: path to stdout file.
        :param stderr: path to stderr file.
        """
        self.stdin = stdin
        self.stdout = stdout
        self.stderr = stderr

    @staticmethod
    def is_file_open(path):
        """
        Determine process which open file.

        :param path: Path to file.
        :return: [[pid,mode], ... ].
        """
        opens = []
        pids = os.listdir('/proc')
        for pid in sorted(pids):
            try:
                int(pid)
            except ValueError:
                continue
            fd_dir = os.path.join('/proc', pid, 'fd')
            try:
                for filepath in os.listdir(fd_dir):
                    try:
                        p = os.path.join(fd_dir, filepath)
                        link = os.readlink(os.path.join(fd_dir, filepath))
                        if link == path:
                            mode = os.lstat(p).st_mode
                            opens.append([pid, mode])
                    except OSError:
                        continue
            except OSError as e:
                if e.errno == 2:
                    continue
                raise
        return opens

    def daemonize(self):
        """
        Run guest as a daemon.
        """
        try:
            pid = os.fork()
            if pid > 0:
                return False
        except OSError as e:
            sys.stderr.write("Daemonize failed: %s\n" % (e))
            sys.exit(1)

        os.chdir("/")
        os.setsid()
        os.umask(0)

        try:
            pid = os.fork()
            if pid > 0:
                sys.exit(0)
        except OSError as e:
            sys.stderr.write("Daemonize failed: %s\n" % (e))
            sys.exit(1)

        sys.stdout.flush()
        sys.stderr.flush()
        si = open(self.stdin, 'r')
        so = open(self.stdout, 'w')
        se = open(self.stderr, 'w')

        os.dup2(si.fileno(), sys.stdin.fileno())
        os.dup2(so.fileno(), sys.stdout.fileno())
        os.dup2(se.fileno(), sys.stderr.fileno())

        si.close()
        so.close()
        se.close()

        sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 1)
        sys.stderr = os.fdopen(sys.stderr.fileno(), 'w', 1)
        return True

    def start(self):
        """
        Start the daemon

        :return: PID of daemon.
        """
        # Check for a pidfile to see if the daemon already runs
        openers = self.is_file_open(self.stdout)
        rundaemon = False
        if len(openers) > 0:
            for i in openers:
                if i[1] & stat.S_IWUSR:
                    rundaemon = True
                    openers.remove(i)
            if len(openers) > 0:
                for i in openers:
                    os.kill(int(i[0]), 9)
        time.sleep(0.3)

        # Start the daemon
        if not rundaemon:
            if self.daemonize():
                self.run()

    def run(self):
        """
        Run guest main thread
        """
        global virt
        global exiting
        virt = VirtioGuestPosix()
        slave = Thread(target=worker, args=(virt,))
        slave.start()
        signal.signal(signal.SIGUSR1, sigusr_handler)
        signal.signal(signal.SIGALRM, sigusr_handler)
        while not exiting:
            signal.alarm(1)
            signal.pause()
            catch = virt.catching_signal()
            if catch:
                signal.signal(signal.SIGIO, virt)
            elif catch is False:
                signal.signal(signal.SIGIO, signal.SIG_DFL)
            if catch is not None:
                virt.use_config.set()
        print("PASS: guest_exit")
        sys.exit(0)


def main():
    """
    Main function for OS Linux with infinite loop to catch signal from system.
    """
    stdin = "/tmp/guest_daemon_pi"
    stdout = "/tmp/guest_daemon_po"
    stderr = "/tmp/guest_daemon_pe"

    for f in [stdin, stdout, stderr]:
        try:
            os.mkfifo(f)
        except OSError as e:
            if e.errno == 17:
                pass

    daemon = Daemon(stdin,
                    stdout,
                    stderr)
    daemon.start()

    d_stdin = os.open(stdin, os.O_WRONLY)
    d_stdout = os.open(stdout, os.O_RDONLY)
    d_stderr = os.open(stderr, os.O_RDONLY)

    s_stdin = sys.stdin.fileno()
    s_stdout = sys.stdout.fileno()
    s_stderr = sys.stderr.fileno()

    pid = filter(lambda x: x[0] != str(os.getpid()),
                 daemon.is_file_open(stdout))
    pid = list(pid)[0][0]

    print("PASS: Start")

    while 1:
        ret = select.select([d_stderr,
                             d_stdout,
                             s_stdin],
                            [], [], 1.0)
        if s_stdin in ret[0]:
            os.write(d_stdin, os.read(s_stdin, 1))
        if d_stdout in ret[0]:
            os.write(s_stdout, os.read(d_stdout, 1024))
        if d_stderr in ret[0]:
            os.write(s_stderr, os.read(d_stderr, 1024))
        if not os.path.exists("/proc/" + pid):
            sys.exit(0)

    os.close(d_stdin)
    os.close(d_stdout)
    os.close(d_stderr)


def main_nt():
    """
    Main function for Windows NT with infinite loop.
    """
    global virt
    global exiting
    virt = VirtioGuestNt()
    print("PASS: Start")
    sys.stdout.flush()
    while not exiting:
        try:
            exec(input())
        except Exception:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            print("On Guest exception from: \n" + "".join(
                traceback.format_exception(exc_type,
                                           exc_value,
                                           exc_traceback)))
            print("FAIL: Guest command exception.")
        sys.stdout.flush()
    print("PASS: guest_exit")
    sys.exit(0)


if __name__ == "__main__":
    if (len(sys.argv) > 1) and (sys.argv[1] == "-c"):
        compile_script()

    if os_linux:    # Linux
        main()
    else:   # Windows
        main_nt()



#!/usr/bin/python
"""
Library used to provide the appropriate data dir for virt test.
"""
import inspect
import logging
import os
import glob
import shutil
import stat

import pkg_resources

from avocado.core import data_dir
from avocado.utils import distro
from avocado.utils import path as utils_path

from virttest.compat import get_settings_value

from six.moves import xrange

BASE_BACKEND_DIR = pkg_resources.resource_filename('virttest', 'backends')
TEST_PROVIDERS_DIR = pkg_resources.resource_filename('virttest', 'test-providers.d')
SHARED_DIR = pkg_resources.resource_filename('virttest', 'shared')
DEPS_DIR = os.path.join(SHARED_DIR, 'deps')
BASE_DOWNLOAD_DIR = os.path.join(SHARED_DIR, 'downloads')

DATA_DIR = os.path.join(data_dir.get_data_dir(), 'avocado-vt')
DOWNLOAD_DIR = os.path.join(DATA_DIR, 'downloads')
BACKING_DATA_DIR = None


class MissingDepsDirError(Exception):
    pass


class UnknownBackendError(Exception):

    def __init__(self, backend):
        self.backend = backend

    def __str__(self):
        return ("Virt Backend %s is not currently supported by avocado-vt. "
                "Check for typos and the list of supported backends" %
                self.backend)


class SubdirList(list):

    """
    List of all non-hidden subdirectories beneath basedir
    """

    def __in_filter__(self, item):
        if self.filterlist:
            for _filter in self.filterlist:
                if item.count(str(_filter)):
                    return True
            return False
        else:
            return False

    def __set_initset__(self):
        for dirpath, dirnames, filenames in os.walk(self.basedir):
            del filenames  # not used
            # Don't modify list while in use
            del_list = []
            for _dirname in dirnames:
                if _dirname.startswith('.') or self.__in_filter__(_dirname):
                    # Don't descend into filtered or hidden directories
                    del_list.append(_dirname)
                else:
                    self.initset.add(os.path.join(dirpath, _dirname))
            # Remove items in del_list from dirnames list
            for _dirname in del_list:
                del dirnames[dirnames.index(_dirname)]

    def __init__(self, basedir, filterlist=None):
        self.basedir = os.path.abspath(str(basedir))
        self.initset = set([self.basedir])  # enforce unique items
        self.filterlist = filterlist
        self.__set_initset__()
        super(SubdirList, self).__init__(self.initset)


class SubdirGlobList(SubdirList):

    """
    List of all files matching glob in all non-hidden basedir subdirectories
    """

    def __initset_to_globset__(self):
        globset = set()
        for dirname in self.initset:  # dirname is absolute
            pathname = os.path.join(dirname, self.globstr)
            for filepath in glob.glob(pathname):
                if not self.__in_filter__(filepath):
                    globset.add(filepath)
        self.initset = globset

    def __set_initset__(self):
        super(SubdirGlobList, self).__set_initset__()
        self.__initset_to_globset__()

    def __init__(self, basedir, globstr, filterlist=None):
        self.globstr = str(globstr)
        super(SubdirGlobList, self).__init__(basedir, filterlist)


def get_backing_data_dir():
    return DATA_DIR


BACKING_DATA_DIR = get_backing_data_dir()


def get_root_dir():
    return os.path.dirname(BASE_BACKEND_DIR)


def get_data_dir():
    return DATA_DIR


def get_shared_dir():
    return SHARED_DIR


def get_base_backend_dir():
    return BASE_BACKEND_DIR


def get_local_backend_dir():
    return os.path.join(get_data_dir(), 'backends')


def get_backend_dir(backend_type):
    if backend_type not in os.listdir(BASE_BACKEND_DIR):
        raise UnknownBackendError(backend_type)
    return os.path.join(get_local_backend_dir(), backend_type)


def get_backend_cfg_path(backend_type, cfg_basename):
    return os.path.join(get_backend_dir(backend_type), 'cfg', cfg_basename)


def get_deps_dir(target=None):
    """
    For a given test provider, report the appropriate deps dir.

    The little inspect trick is used to avoid callers having to do
    sys.modules[] tricks themselves.

    :param target: File we want in deps folder. Will return the path to the
                   target if set and available. Or will only return the path
                   to dep folder.
    """
    # Get the frame that called this function
    frame = inspect.stack()[1]
    # This is the module that called the function
    # With the module path, we can keep searching with a parent dir with 'deps'
    # in it, which should be the correct deps directory.
    try:
        module = inspect.getmodule(frame[0])
        path = os.path.dirname(module.__file__)
    except TypeError:
        path = os.path.dirname(frame[1])
    nesting_limit = 10
    for index in xrange(nesting_limit):
        files = os.listdir(path)
        origin_path = ""
        if 'shared' in files:
            origin_path = path
            path = os.path.join(path, 'shared')
            files = os.listdir(path)
        if 'deps' in files:
            deps = os.path.join(path, 'deps')
            if target:
                if target in os.listdir(deps):
                    return os.path.join(deps, target)
            else:
                return deps
        if '.git' in os.listdir(path):
            raise MissingDepsDirError("Could not find specified deps dir for "
                                      "git repo %s" % path)
        if origin_path:
            path = origin_path
        path = os.path.dirname(path)
    raise MissingDepsDirError("Could not find specified deps dir after "
                              "looking %s parent directories" %
                              nesting_limit)


def get_tmp_dir(public=True):
    """
    Get the most appropriate tmp dir location.

    :param public: If public for all users' access
    """
    persistent_dir = get_settings_value('vt.common', 'tmp_dir',
                                        default="")
    if persistent_dir != "":
        return persistent_dir
    tmp_dir = None
    # apparmor deny /tmp/* /var/tmp/* and cause failure across tests
    # it is better to handle here
    if distro.detect().name == 'Ubuntu':
        tmp_dir = "/var/lib/libvirt/images"
        if not utils_path.usable_rw_dir(tmp_dir):
            logging.warning("Unable to write in '/var/lib/libvirt/images' "
                            "on Ubuntu, apparmor might complain...")
            tmp_dir = None
    tmp_dir = data_dir.get_tmp_dir(basedir=tmp_dir)
    if public:
        tmp_dir_st = os.stat(tmp_dir)
        os.chmod(tmp_dir, tmp_dir_st.st_mode | stat.S_IXUSR |
                 stat.S_IXGRP | stat.S_IXOTH | stat.S_IRGRP | stat.S_IROTH)
    return tmp_dir


def get_base_download_dir():
    return BASE_DOWNLOAD_DIR


def get_download_dir():
    return DOWNLOAD_DIR


TEST_PROVIDERS_DOWNLOAD_DIR = os.path.join(get_data_dir(),
                                           'virttest',
                                           'test-providers.d',
                                           'downloads')


def get_base_test_providers_dir():
    return TEST_PROVIDERS_DIR


def get_test_providers_dir():
    """
    Return the base test providers dir (at the moment, test-providers.d).
    """
    return os.path.dirname(TEST_PROVIDERS_DOWNLOAD_DIR)


def get_test_provider_dir(provider):
    """
    Return a specific test providers dir, inside the base dir.
    """
    return os.path.join(TEST_PROVIDERS_DOWNLOAD_DIR, provider)


def clean_tmp_files():
    tmp_dir = get_tmp_dir()
    if os.path.isdir(tmp_dir):
        hidden_paths = glob.glob(os.path.join(tmp_dir, ".??*"))
        paths = glob.glob(os.path.join(tmp_dir, "*"))
        for path in paths + hidden_paths:
            shutil.rmtree(path, ignore_errors=True)


if __name__ == '__main__':
    print("root dir:         " + get_root_dir())
    print("tmp dir:          " + get_tmp_dir())
    print("data dir:         " + DATA_DIR)
    print("deps dir:         " + DEPS_DIR)
    print("backing data dir: " + BACKING_DATA_DIR)
    print("test providers dir: " + TEST_PROVIDERS_DIR)



"""
Higher order classes for Libvirt Sandbox Service (lxc) service container testing
"""

from avocado.utils import process
from avocado.utils import service

from . import lvsb_base
from . import virsh


class SandboxService(object):

    """
    Management for a single new/existing sandboxed service
    """

    def __init__(self, params, service_name, uri='lxc:///'):
        """Initialize connection to sandbox service with name and parameters"""
        # Intended workflow is:
        #   Use virt-sandbox-service for create/destroy
        #   Use service/systemd for runtime management
        #   Use virsh for list/edit/modify manipulation
        self.virsh = virsh.Virsh(uri=uri, ignore_status=True)
        self.command = lvsb_base.SandboxCommandBase(params, service_name)
        self.command.BINARY_PATH_PARAM = params.get('virt_sandbox_service_binary',
                                                    "virt-sandbox-service")
        self.command.add_optarg('--connect', uri)
        # We need to pass self.service_name to service.Factory.create_service to
        # create a service. Then we will get a SpecificServiceManager object as
        # self.service. But SpecificServiceManager is not pickleable, save init
        # args here.
        self._run = process.run
        self.service = service.Factory.create_service(self.service_name,
                                                      run=self._run)
        # make self.start() --> self.service.start()
        self._bind_service_commands()

    def _bind_service_commands(self):
        """Setup service methods locally for __init__ and __setstate__"""
        for command in service.COMMANDS:
            # Use setattr to keep pylint quiet
            setattr(self, command, getattr(self.service, command))

    def __getstate__(self):
        """Serialize instance for pickling"""
        # SandboxCommandBase is directly pickleable
        return {'command': self.command, 'run': self._run, 'virsh': dict(virsh)}

    def __setstate__(self, state):
        """Actualize instance from state"""
        # virsh is it's own dict of init params
        self.virsh = virsh.Virsh(**state['virsh'])
        # already used it's own get/sets state methods when unpickling state
        self.command = state['command']
        # Recreate SpecificServiceManager from the init args
        self._run = state['run']
        self.service = service.Factory.create_service(self.service_name,
                                                      run=self._run)
        self._bind_service_commands()

    # Enforce read-only at all levels
    @property
    def service_name(self):
        return self.command.name

    # property accessor functions must be defined before naming attribute
    def __get_uri__(self):
        return self.virsh.uri

    def __set_uri__(self, uri):
        self.virsh.uri = uri

    def __del_uri__(self):
        # Virsh class interface insists this attribute exist, but can be None
        self.virsh.uri = None

    # Property definition must follow accessor definitions
    uri = property(__get_uri__, __set_uri__, __del_uri__)

    def create(self):
        return self.command.run(extra='create')

    def destroy(self):
        return self.command.run(extra='destroy')

    # Specialized list calls can just call self.virsh.dom_list() directly
    @property  # behave like attribute to make value-access easier
    def list(self):
        """
        Return list of dictionaries mapping column names to values
        """
        # For simple callers, just return list of names to be convenient
        cmdresult = self.virsh.dom_list()  # uri is passed automatically
        result = []
        column_names = None  # scope outside loop
        for lineno, line in cmdresult.stdout_text.strip():
            if lineno == 0:
                column_names = line.strip().split()
                assert len(column_names) > 2
            else:
                assert column_names is not None
                # raises exception when column_names & value count mismatch
                items = [(column_names[index].lower(), value.lower())
                         for index, value in line.strip().split()]
                # combine [('id',99), ('name', 'foobar'), ('state', 'running')]
                result.append(dict(items))
        return result

    # Specialized list calls can just call self.virsh.dom_list() directly
    @property  # behave like attribute for easy passing to XML handling methods
    def xmlstr(self):
        result = self.virsh.dumpxml(self.service_name)
        return result.stdout_text.strip()



"""
Module simplifying manipulation of XML described at
http://libvirt.org/formatnetwork.html
"""

import logging
import six

from virttest import xml_utils
from virttest.libvirt_xml import base, xcepts, accessors
from virttest.libvirt_xml.devices import librarian


class RangeList(list):

    """
    A list of start & end address tuples
    """

    def __init__(self, iterable=None):
        """
        Initialize from list/tuple of two-item tuple start/end address strings
        """
        x_str = "iterable must contain two-item tuples of start/end addresses"
        newone = []
        for item in iterable:
            if not issubclass(type(item), tuple):
                raise xcepts.LibvirtXMLError(x_str)
            if len(item) != 2:
                raise xcepts.LibvirtXMLError(x_str)
            # Assume strings will be validated elsewhere
            newone.append(tuple(item))
        super(RangeList, self).__init__(newone)

    def append_to_element(self, element):
        """
        Adds range described by instance to ElementTree.element
        """
        if not issubclass(type(element), xml_utils.ElementTree.Element):
            raise ValueError(
                "Element is not a ElementTree.Element or subclass")
        for start, end in self:
            serange = {'start': start, 'end': end}
            element.append(xml_utils.ElementTree.Element('range', serange))


class IPXML(base.LibvirtXMLBase):

    """
    IP address block, optionally containing DHCP range information

    Properties:
        dhcp_ranges: Dict. keys: start, end
        host_attr: host mac, name and ip information
        address: string IP address
        netmask: string IP's netmask
    """

    __slots__ = ('dhcp_ranges', 'address', 'netmask', 'hosts',
                 'family', 'prefix', 'tftp_root', 'dhcp_bootp')

    def __init__(self, address='192.168.122.1', netmask='255.255.255.0',
                 ipv6=False, virsh_instance=base.virsh):
        """
        Create new IPXML instance based on address/mask
        """
        accessors.XMLAttribute(
            'address', self, parent_xpath='/', tag_name='ip',
            attribute='address')
        accessors.XMLAttribute(
            'netmask', self, parent_xpath='/', tag_name='ip',
            attribute='netmask')
        accessors.XMLAttribute(
            'family', self, parent_xpath='/', tag_name='ip',
            attribute='family')
        accessors.XMLAttribute(
            'prefix', self, parent_xpath='/', tag_name='ip',
            attribute='prefix')
        accessors.XMLAttribute(
            'tftp_root', self, parent_xpath='/', tag_name='tftp',
            attribute='root')
        accessors.XMLAttribute(
            'dhcp_bootp', self, parent_xpath='/dhcp', tag_name='bootp',
            attribute='file')
        accessors.XMLElementDict('dhcp_ranges', self,
                                 parent_xpath='/dhcp',
                                 tag_name='range')
        accessors.XMLElementList('hosts', self, parent_xpath='/dhcp',
                                 marshal_from=self.marshal_from_host,
                                 marshal_to=self.marshal_to_host)
        super(IPXML, self).__init__(virsh_instance=virsh_instance)
        if ipv6:
            self.xml = u"<ip address='%s'></ip>" % address
        else:
            self.xml = u"<ip address='%s' netmask='%s'></ip>" % (address, netmask)

    @staticmethod
    def marshal_from_host(item, index, libvirtxml):
        """Convert a dictionary into a tag + attributes"""
        del index           # not used
        del libvirtxml      # not used
        if not isinstance(item, dict):
            raise xcepts.LibvirtXMLError("Expected a dictionary of host "
                                         "attributes, not a %s"
                                         % str(item))
        return ('host', dict(item))  # return copy of dict, not reference

    @staticmethod
    def marshal_to_host(tag, attr_dict, index, libvirtxml):
        """Convert a tag + attributes into a dictionary"""
        del index                    # not used
        del libvirtxml               # not used
        if tag != 'host':
            return None              # skip this one
        return dict(attr_dict)       # return copy of dict, not reference


class DNSXML(base.LibvirtXMLBase):

    """
    IP address block, optionally containing DHCP range information

    Properties:
        txt:
            Dict. keys: name, value
        forwarder:
            List
        srv:
            Dict. keys: service, protocol, domain,
            tartget, port, priority, weight
        hosts:
            List of host name
    """

    __slots__ = ('dns_forward', 'txt', 'forwarders', 'srv',
                 'host')

    def __init__(self, virsh_instance=base.virsh):
        """
        Create new IPXML instance based on address/mask
        """
        accessors.XMLElementDict('txt', self,
                                 parent_xpath='/',
                                 tag_name='txt')
        accessors.XMLElementDict('srv', self,
                                 parent_xpath='/',
                                 tag_name='srv')
        accessors.XMLElementList('forwarders', self, parent_xpath='/',
                                 marshal_from=self.marshal_from_forwarder,
                                 marshal_to=self.marshal_to_forwarder)
        accessors.XMLAttribute('dns_forward', self, parent_xpath='/',
                               tag_name='dns', attribute='forwardPlainNames')
        accessors.XMLElementNest('host', self, parent_xpath='/',
                                 tag_name='host', subclass=DNSXML.HostXML,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        super(DNSXML, self).__init__(virsh_instance=virsh_instance)
        self.xml = u"<dns></dns>"

    class HostnameXML(base.LibvirtXMLBase):

        """
        Hostname element of dns
        """

        __slots__ = ('hostname',)

        def __init__(self, virsh_instance=base.virsh):
            """
            Create new HostnameXML instance
            """
            accessors.XMLElementText('hostname', self, parent_xpath='/',
                                     tag_name='hostname')
            super(DNSXML.HostnameXML, self).__init__(
                virsh_instance=virsh_instance)
            self.xml = '<hostname/>'

    class HostXML(base.LibvirtXMLBase):

        """
        Hostname element of dns
        """

        __slots__ = ('host_ip', 'hostnames',)

        def __init__(self, virsh_instance=base.virsh):
            """
            Create new TimerXML instance
            """
            accessors.XMLAttribute('host_ip', self, parent_xpath='/',
                                   tag_name='host', attribute='ip')
            accessors.XMLElementList('hostnames', self, parent_xpath='/',
                                     marshal_from=self.marshal_from_hostname,
                                     marshal_to=self.marshal_to_hostname)
            super(DNSXML.HostXML, self).__init__(virsh_instance=virsh_instance)
            self.xml = '<host/>'

        @staticmethod
        def marshal_from_hostname(item, index, libvirtxml):
            """Convert a HostnameXML instance into a tag + attributes"""
            del index           # not used
            del libvirtxml      # not used
            if isinstance(item, six.string_types):
                return ("hostname", {}, item)
            else:
                raise xcepts.LibvirtXMLError("Expected a str attributes,"
                                             " not a %s" % str(item))

        @staticmethod
        def marshal_to_hostname(tag, attr, index, libvirtxml, text):
            """Convert a tag + attributes into a HostnameXML instance"""
            del attr                     # not used
            del index                    # not used
            if tag != 'hostname':
                return None     # Don't convert this item
            newone = DNSXML.HostnameXML(virsh_instance=libvirtxml.virsh)
            newone.hostname = text
            return newone

    def new_host(self, **dargs):
        """
        Return a new disk IOTune instance and set properties from dargs
        """
        new_one = DNSXML.HostXML(virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    @staticmethod
    def marshal_from_forwarder(item, index, libvirtxml):
        """Convert a dictionary into a tag + attributes"""
        del index           # not used
        del libvirtxml      # not used
        if not isinstance(item, dict):
            raise xcepts.LibvirtXMLError("Expected a dictionary of host "
                                         "attributes, not a %s"
                                         % str(item))
        return ('forwarder', dict(item))  # return copy of dict, not reference

    @staticmethod
    def marshal_to_forwarder(tag, attr_dict, index, libvirtxml):
        """Convert a tag + attributes into a dictionary"""
        del index                    # not used
        del libvirtxml               # not used
        if tag != 'forwarder':
            return None              # skip this one
        return dict(attr_dict)       # return copy of dict, not reference


class PortgroupXML(base.LibvirtXMLBase):

    """
    Accessor methods for PortgroupXML class in NetworkXML.

    Properties:
        name:
            string, operates on 'name' attribute of portgroup tag
        default:
            string of yes or no, operates on 'default' attribute of
            portgroup tag
        virtualport_type:
            string, operates on 'type' attribute of virtualport tag in
            portgroup.
        bandwidth_inbound:
            dict, operates on inbound tag in bandwidth which is child
            of portgroup.
        bandwidth_outbound:
            dict, operates on outbound tag in bandwidth which is child
            of portgroup.
        vlan_tag:
            dict, operates on vlan tag of portgroup
    """

    __slots__ = ('name', 'default', 'virtualport_type',
                 'bandwidth_inbound', 'bandwidth_outbound',
                 'vlan_tag')

    def __init__(self, virsh_instance=base.virsh):
        """
        Create new PortgroupXML instance.
        """
        accessors.XMLAttribute('name', self, parent_xpath='/',
                               tag_name='portgroup', attribute='name')
        accessors.XMLAttribute('default', self, parent_xpath='/',
                               tag_name='portgroup', attribute='default')
        accessors.XMLAttribute('virtualport_type', self, parent_xpath='/',
                               tag_name='virtualport', attribute='type')
        accessors.XMLElementDict('bandwidth_inbound', self,
                                 parent_xpath='/bandwidth',
                                 tag_name='inbound')
        accessors.XMLElementDict('bandwidth_outbound', self,
                                 parent_xpath='/bandwidth',
                                 tag_name='outbound')
        accessors.XMLElementDict('vlan_tag', self,
                                 parent_xpath='/vlan',
                                 tag_name='tag')
        super(PortgroupXML, self).__init__(virsh_instance=virsh_instance)
        self.xml = u"<portgroup></portgroup>"


class NetworkXMLBase(base.LibvirtXMLBase):

    """
    Accessor methods for NetworkXML class.

    Properties:
        name:
            string, operates on XML name tag
        uuid:
            string, operates on uuid tag
        mac:
            string, operates on address attribute of mac tag
        ip:
            string operate on ip/dhcp ranges as IPXML instances
        forward:
            dict, operates on forward tag
        forward_interface:
            list, operates on forward/interface tag
        nat_port:
            dict, operates on nat tag
        bridge:
            dict, operates on bridge attributes
        routes:
            list, operates on route tag.
        virtualport_type:
            string, operates on 'type' attribute of virtualport tag.
        bandwidth_inbound:
            dict, operates on inbound under bandwidth.
        bandwidth_outbound:
            dict, operates on outbound under bandwidth.
        portgroup:
            PortgroupXML instance to access portgroup tag.
        domain_name:
            string, operates on name attribute of domain tag
        dns:
            DNSXML instance to access dns tag.

        defined:
            virtual boolean, callout to virsh methods
        get:
            True if libvirt knows network name
        set:
            True defines network, False undefines to libvirt
        del:
            Undefines network to libvirt

        active:
            virtual boolean, callout to virsh methods
        get:
            True if network is active to libvirt
        set:
            True activates network, False deactivates to libvirt
        del:
            Deactivates network to libvirt

        autostart:
            virtual boolean, callout to virsh methods
        get:
            True if libvirt autostarts network with same name
        set:
            True to set autostart, False to unset to libvirt
        del:
            Unset autostart to libvirt

        persistent:
            virtual boolean, callout to virsh methods
        get:
            True if network was defined, False if only created.
        set:
            Same as defined property
        del:
            Same as defined property
    """

    __slots__ = ('name', 'uuid', 'bridge', 'defined', 'active',
                 'autostart', 'persistent', 'forward', 'mac', 'ip',
                 'bandwidth_inbound', 'bandwidth_outbound', 'portgroup',
                 'dns', 'domain_name', 'nat_port', 'forward_interface',
                 'routes', 'virtualport_type', 'vf_list', 'driver', 'pf',
                 'mtu', 'connection', 'port', 'nat_attrs')

    __uncompareable__ = base.LibvirtXMLBase.__uncompareable__ + (
        'defined', 'active',
        'autostart', 'persistent')

    __schema_name__ = "network"

    def __init__(self, virsh_instance=base.virsh):
        accessors.XMLAttribute('connection', self, parent_xpath='/',
                               tag_name='network', attribute='connections')
        accessors.XMLElementText('name', self, parent_xpath='/',
                                 tag_name='name')
        accessors.XMLElementText('uuid', self, parent_xpath='/',
                                 tag_name='uuid')
        accessors.XMLAttribute('mac', self, parent_xpath='/',
                               tag_name='mac', attribute='address')
        accessors.XMLElementDict('forward', self, parent_xpath='/',
                                 tag_name='forward')
        accessors.XMLElementList('forward_interface', self, parent_xpath='/forward',
                                 marshal_from=self.marshal_from_forward_iface,
                                 marshal_to=self.marshal_to_forward_iface)
        accessors.XMLElementDict('nat_attrs', self, parent_xpath='/forward',
                                 tag_name='nat')
        accessors.XMLElementList('vf_list', self,
                                 parent_xpath='/forward',
                                 marshal_from=self.marshal_from_address,
                                 marshal_to=self.marshal_to_address)
        accessors.XMLElementDict('driver', self, parent_xpath='/',
                                 tag_name='driver')
        accessors.XMLElementDict('pf', self, parent_xpath='/forward',
                                 tag_name='pf')
        accessors.XMLElementDict('nat_port', self, parent_xpath='/forward/nat',
                                 tag_name='port')
        accessors.XMLElementDict('bridge', self, parent_xpath='/',
                                 tag_name='bridge')
        accessors.XMLElementDict('bandwidth_inbound', self,
                                 parent_xpath='/bandwidth',
                                 tag_name='inbound')
        accessors.XMLElementDict('bandwidth_outbound', self,
                                 parent_xpath='/bandwidth',
                                 tag_name='outbound')
        accessors.XMLElementDict('port', self, parent_xpath='/',
                                 tag_name='port')
        accessors.XMLAttribute("mtu", self, parent_xpath='/',
                               tag_name='mtu',  attribute='size')
        accessors.XMLAttribute('domain_name', self, parent_xpath='/',
                               tag_name='domain', attribute='name')
        accessors.XMLElementNest('dns', self, parent_xpath='/',
                                 tag_name='dns', subclass=DNSXML,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        accessors.XMLElementList('routes', self, parent_xpath='/',
                                 marshal_from=self.marshal_from_route,
                                 marshal_to=self.marshal_to_route)
        accessors.XMLAttribute('virtualport_type', self, parent_xpath='/',
                               tag_name='virtualport', attribute='type')
        super(NetworkXMLBase, self).__init__(virsh_instance=virsh_instance)

    Address = librarian.get('address')

    def new_vf_address(self, **dargs):
        """
        Return a new interface Address instance and set properties from dargs
        """
        new_one = self.Address("pci", virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    @staticmethod
    def marshal_from_address(item, index, libvirtxml):
        """Convert an Address instance into tag + attributes"""
        root = item.xmltreefile.getroot()
        if root.tag == 'address':
            return (root.tag, dict(list(root.items())))
        else:
            raise xcepts.LibvirtXMLError("Expected a list of address "
                                         "instances, not a %s" % str(item))

    @staticmethod
    def marshal_to_address(self, tag, attr_dict, index, libvirtxml):
        """Convert a tag + attributes into an Address instance"""
        if not tag == 'address':
            return None
        newone = self.new_vf_address(attr_dict)
        return newone

    def __check_undefined__(self, errmsg):
        if not self.defined:
            raise xcepts.LibvirtXMLError(errmsg)

    def get_defined(self):
        """
        Accessor for 'define' property - does this name exist in network list
        """
        params = {'only_names': True, 'virsh_instance': self.virsh}
        return self.name in self.virsh.net_state_dict(**params)

    def set_defined(self, value):
        """Accessor method for 'define' property, set True to define."""
        if not self.__super_get__('INITIALIZED'):
            pass  # do nothing
        value = bool(value)
        if value:
            self.virsh.net_define(self.xml)  # send it the filename
        else:
            del self.defined

    def del_defined(self):
        """Accessor method for 'define' property, undefines network"""
        self.__check_undefined__("Cannot undefine non-existant network")
        self.virsh.net_undefine(self.name)

    def get_active(self):
        """Accessor method for 'active' property (True/False)"""
        state_dict = self.virsh.net_state_dict(virsh_instance=self.virsh)
        try:
            active_state = state_dict[self.name]['active']
        except KeyError:
            return False
        return active_state

    def set_active(self, value):
        """Accessor method for 'active' property, sets network active"""
        if not self.__super_get__('INITIALIZED'):
            pass  # do nothing
        self.__check_undefined__("Cannot activate undefined network")
        value = bool(value)
        if value:
            if not self.active:
                self.virsh.net_start(self.name)
            else:
                pass  # don't activate twice
        else:
            if self.active:
                del self.active
            else:
                pass  # don't deactivate twice

    def del_active(self):
        """Accessor method for 'active' property, stops network"""
        if self.active:
            self.virsh.net_destroy(self.name)
        else:
            pass  # don't destroy twice

    def get_autostart(self):
        """Accessor method for 'autostart' property, True if set"""
        self.__check_undefined__("Cannot determine autostart for undefined "
                                 "network")
        state_dict = self.virsh.net_state_dict(virsh_instance=self.virsh)
        return state_dict[self.name]['autostart']

    def set_autostart(self, value):
        """Accessor method for 'autostart' property, sets/unsets autostart"""
        if not self.__super_get__('INITIALIZED'):
            pass  # do nothing
        self.__check_undefined__("Cannot set autostart for undefined network")
        value = bool(value)
        if value:
            if not self.autostart:
                self.virsh.net_autostart(self.name)
            else:
                pass  # don't set autostart twice
        else:
            if self.autostart:
                del self.autostart
            else:
                pass  # don't unset autostart twice

    def del_autostart(self):
        """Accessor method for 'autostart' property, unsets autostart"""
        if not self.defined:
            raise xcepts.LibvirtXMLError("Can't autostart nonexistant network")
        self.virsh.net_autostart(self.name, "--disable")

    def get_persistent(self):
        """Accessor method for 'persistent' property"""
        state_dict = self.virsh.net_state_dict(virsh_instance=self.virsh)
        return state_dict[self.name]['persistent']

    # Copy behavior for consistency
    set_persistent = set_defined
    del_persistent = del_defined

    def get_ip(self):
        xmltreefile = self.__dict_get__('xml')
        try:
            ip_root = xmltreefile.reroot('/ip')
        except KeyError as detail:
            raise xcepts.LibvirtXMLError(detail)
        ipxml = IPXML(virsh_instance=self.__dict_get__('virsh'))
        ipxml.xmltreefile = ip_root
        return ipxml

    def set_ip(self, value):
        if not issubclass(type(value), IPXML):
            raise xcepts.LibvirtXMLError("value must be a IPXML or subclass")
        xmltreefile = self.__dict_get__('xml')
        # IPXML root element is whole IP element tree
        root = xmltreefile.getroot()
        root.append(value.xmltreefile.getroot())
        xmltreefile.write()

    def del_ip(self):
        xmltreefile = self.__dict_get__('xml')
        element = xmltreefile.find('/ip')
        if element is not None:
            xmltreefile.remove(element)
            xmltreefile.write()

    def del_element(self, element='', index=0):
        """
        Delete element from network xml

        :param element: xpath of element like '/ip/dhcp'
        :param index: index of element that want to delete
        """
        xmltreefile = self.__dict_get__('xml')
        try:
            del_elem = xmltreefile.findall(element)[index]
        except IndexError as detail:
            del_elem = None
            logging.warning(detail)
        if del_elem is not None:
            xmltreefile.remove(del_elem)
            xmltreefile.write()

    def get_portgroup(self):
        try:
            portgroup_root = self.xmltreefile.reroot('/portgroup')
        except KeyError as detail:
            raise xcepts.LibvirtXMLError(detail)
        portgroup_xml = PortgroupXML(virsh_instance=self.__dict_get__('virsh'))
        portgroup_xml.xmltreefile = portgroup_root
        return portgroup_xml

    def set_portgroup(self, value):
        if not issubclass(type(value), PortgroupXML):
            raise xcepts.LibvirtXMLError("value must be a PortgroupXML"
                                         "instance or subclass.")
        root = self.xmltreefile.getroot()
        root.append(value.xmltreefile.getroot())
        self.xmltreefile.write()

    def del_portgroup(self):
        element = self.xmltreefile.find("/portgroup")
        if element is not None:
            self.xmltreefile.remove(element)
            self.xmltreefile.write()

    def get_interface_connection(self):
        try:
            ifaces = self.xmltreefile.findall("/forward/interface")
        except KeyError as detail:
            raise xcepts.LibvirtXMLError(detail)
        iface_conn = []
        for iface in ifaces:
            try:
                iface_conn.append(iface.get("connections"))
            except KeyError:
                pass
        return iface_conn

    def new_dns(self, **dargs):
        """
        Return a new dns instance and set properties from dargs
        """
        new_one = DNSXML(virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    @staticmethod
    def marshal_from_forward_iface(item, index, libvirtxml):
        """Convert a dictionary into a tag + attributes"""
        del index           # not used
        del libvirtxml      # not used
        if not isinstance(item, dict):
            raise xcepts.LibvirtXMLError("Expected a dictionary of interface "
                                         "attributes, not a %s"
                                         % str(item))
        return ('interface', dict(item))  # return copy of dict, not reference

    @staticmethod
    def marshal_to_forward_iface(tag, attr_dict, index, libvirtxml):
        """Convert a tag + attributes into a dictionary"""
        del index                    # not used
        del libvirtxml               # not used
        if tag != 'interface':
            return None              # skip this one
        return dict(attr_dict)       # return copy of dict, not reference

    @staticmethod
    def marshal_from_route(item, index, libvirtxml):
        """Convert a dictionary into a tag + attributes"""
        del index           # not used
        del libvirtxml      # not used
        if not isinstance(item, dict):
            raise xcepts.LibvirtXMLError("Expected a dictionary of interface "
                                         "attributes, not a %s"
                                         % str(item))
        return ('route', dict(item))  # return copy of dict, not reference

    @staticmethod
    def marshal_to_route(tag, attr_dict, index, libvirtxml):
        """Convert a tag + attributes into a dictionary"""
        del index                    # not used
        del libvirtxml               # not used
        if tag != 'route':
            return None              # skip this one
        return dict(attr_dict)       # return copy of dict, not reference


class NetworkXML(NetworkXMLBase):

    """
    Manipulators of a Virtual Network through it's XML definition.
    """

    __slots__ = []

    def __init__(self, network_name='default', virsh_instance=base.virsh):
        """
        Initialize new instance with empty XML
        """
        super(NetworkXML, self).__init__(virsh_instance=virsh_instance)
        self.xml = u"<network><name>%s</name></network>" % network_name

    @staticmethod  # wraps __new__
    def new_all_networks_dict(virsh_instance=base.virsh):
        """
        Return a dictionary of names to NetworkXML instances for all networks

        :param virsh: virsh module or instance to use
        :return: Dictionary of network name to NetworkXML instance
        """
        result = {}
        # Values should all share virsh property
        new_netxml = NetworkXML(virsh_instance=virsh_instance)
        params = {'only_names': True, 'virsh_instance': virsh_instance}
        networks = list(new_netxml.virsh.net_state_dict(**params).keys())
        for net_name in networks:
            new_copy = new_netxml.copy()
            dump_result = virsh_instance.net_dumpxml(net_name)
            new_copy.xml = dump_result.stdout_text.strip()
            result[net_name] = new_copy
        return result

    @staticmethod
    def new_from_net_dumpxml(network_name, virsh_instance=base.virsh, extra=""):
        """
        Return new NetworkXML instance from virsh net-dumpxml command

        :param network_name: Name of network to net-dumpxml
        :param virsh_instance: virsh module or instance to use
        :return: New initialized NetworkXML instance
        """
        netxml = NetworkXML(virsh_instance=virsh_instance)
        dump_result = virsh_instance.net_dumpxml(network_name, extra)
        netxml['xml'] = dump_result.stdout_text.strip()
        return netxml

    @staticmethod
    def get_uuid_by_name(network_name, virsh_instance=base.virsh):
        """
        Return Network's uuid by Network's name.

        :param network_name: Network's name
        :return: Network's uuid
        """
        network_xml = NetworkXML.new_from_net_dumpxml(network_name,
                                                      virsh_instance)
        return network_xml.uuid

    def debug_xml(self):
        """
        Dump contents of XML file for debugging
        """
        xml = str(self)  # LibvirtXMLBase.__str__ returns XML content
        for debug_line in str(xml).splitlines():
            logging.debug("Network XML: %s", debug_line)

    def state_dict(self):
        """
        Return a dict containing states of active/autostart/persistent

        :return: A dict contains active/autostart/persistent as keys
                 and boolean as values or None if network doesn't exist.
        """
        if self.defined:
            return self.virsh.net_state_dict(virsh_instance=self.virsh)[self.name]

    def create(self, **dargs):
        """
        Adds non-persistant / transient network to libvirt with net-create
        """
        cmd_result = self.virsh.net_create(self.xml, **dargs)
        if cmd_result.exit_status:
            raise xcepts.LibvirtXMLError("Failed to create transient network %s.\n"
                                         "Detail: %s" %
                                         (self.name, cmd_result.stderr))

    def orbital_nuclear_strike(self):
        """It's the only way to really be sure.  Remove all libvirt state"""
        try:
            self['active'] = False  # deactivate (stop) network if active
        except xcepts.LibvirtXMLError as detail:
            # inconsequential, network will be removed
            logging.warning(detail)
        try:
            self['defined'] = False  # undefine (delete) network if persistent
        except xcepts.LibvirtXMLError as detail:
            # network already gone
            logging.warning(detail)

    def exists(self):
        """
        Return True if network already exists.
        """
        cmd_result = self.virsh.net_uuid(self.name)
        return (cmd_result.exit_status == 0)

    def undefine(self):
        """
        Undefine network witch name is self.name.
        """
        self.virsh.net_destroy(self.name)
        cmd_result = self.virsh.net_undefine(self.name)
        if cmd_result.exit_status:
            raise xcepts.LibvirtXMLError("Failed to undefine network %s.\n"
                                         "Detail: %s" %
                                         (self.name,
                                          cmd_result.stderr_text))

    def define(self):
        """
        Define network from self.xml.
        """
        cmd_result = self.virsh.net_define(self.xml)
        if cmd_result.exit_status:
            raise xcepts.LibvirtXMLError("Failed to define network %s.\n"
                                         "Detail: %s" %
                                         (self.name,
                                          cmd_result.stderr_text))

    def start(self):
        """
        Start network with self.virsh.
        """
        cmd_result = self.virsh.net_start(self.name, debug=True)
        if cmd_result.exit_status:
            raise xcepts.LibvirtXMLError("Failed to start network %s.\n"
                                         "Detail: %s" %
                                         (self.name,
                                          cmd_result.stderr_text))

    def sync(self, state=None):
        """
        Make the change of "self" take effect on network.
        Recover network to designated state if option state is set.

        :param state: a boolean dict contains active/persistent/autostart as
                      keys
        """
        if self['defined']:
            if self['active']:
                del self['active']
            if self['defined']:
                del self['defined']

        self['defined'] = True
        if state:
            self['active'] = state['active']
            if not state['persistent']:
                del self['persistent']
            if self.defined:
                self['autostart'] = state['autostart']
        else:
            self['active'] = True
            self['autostart'] = True



import logging
import os
import json
import filecmp

from avocado.utils import process
from avocado.core import exceptions

from virttest import utils_misc
from virttest.libvirt_xml.backup_xml import BackupXML
from virttest.libvirt_xml.checkpoint_xml import CheckpointXML


class BackupError(Exception):

    """
    Class of backup exception
    """

    def __init__(self, *args):
        Exception.__init__(self, *args)


class BackupBeginError(BackupError):

    """
    Class of backup begin exception
    """

    def __init__(self, error_info):
        BackupError.__init__(self, error_info)
        self.error_info = error_info

    def __str__(self):
        return "Backup failed: %s" % self.error_info


class BackupTLSError(BackupError):

    """
    Class of backup TLS exception
    """

    def __init__(self, error_info):
        BackupError.__init__(self, error_info)
        self.error_info = error_info

    def __str__(self):
        return "Backup TLS failure: %s" % self.error_info


def create_checkpoint_xml(cp_params, disk_param_list=None):
    """
    Create a checkpoint xml

    :param cp_params: Params of checkpoint
    :param disk_param_list: A list of disk params used in checkpoint xml
    :return: The checkponit xml
    """
    cp_xml = CheckpointXML()
    cp_name = cp_params.get("checkpoint_name")
    cp_desc = cp_params.get("checkpoint_desc")
    if not cp_name:
        raise exceptions.TestError("Checkpoint name must be provided")
    cp_xml.name = cp_name
    if cp_desc:
        cp_xml.description = cp_desc
    if disk_param_list:
        if not isinstance(disk_param_list, list):
            raise exceptions.TestError("checkpoint disk tags should be defined "
                                       "by a list.")
        cp_xml.disks = disk_param_list
    cp_xml.xmltreefile.write()

    utils_misc.wait_for(lambda: os.path.exists(cp_xml.xml), 5)
    return cp_xml


def create_backup_xml(backup_params, disk_xml_list=None):
    """
    Create a backup xml

    :param backup_params: Params of backup
    :param disk_xml_list: A list of disk params used in backup xml
    :return: The backup xml
    """
    backup_xml = BackupXML()
    backup_mode = backup_params.get("backup_mode")
    backup_incremental = backup_params.get("backup_incremental")
    backup_server = backup_params.get("backup_server")
    if backup_mode:
        backup_xml.mode = backup_mode
    if backup_incremental:
        backup_xml.incremental = backup_incremental
    if backup_server:
        if not isinstance(backup_server, dict):
            raise exceptions.TestError("Backup server tag should be defined by a dict.")
        backup_xml.server = backup_server
    backup_xml.set_disks(disk_xml_list)
    backup_xml.xmltreefile.write()

    utils_misc.wait_for(lambda: os.path.exists(backup_xml.xml), 5)
    return backup_xml


def create_backup_disk_xml(backup_disk_params):
    """
    Create a disk xml which is a subelement of a backup xml

    :param backup_disk_params: Params of disk xml
    :return: The disk xml
    """
    backup_disk_xml = BackupXML.DiskXML()
    disk_name = backup_disk_params.get("disk_name")
    disk_type = backup_disk_params.get("disk_type")
    enable_backup = backup_disk_params.get("enable_backup")
    exportname = backup_disk_params.get("exportname")
    exportbitmap = backup_disk_params.get("exportbitmap")
    backupmode = backup_disk_params.get("backupmode")
    incremental = backup_disk_params.get("incremental")
    backup_target = backup_disk_params.get("backup_target")  # dict
    backup_driver = backup_disk_params.get("backup_driver")  # dict
    backup_scratch = backup_disk_params.get("backup_scratch")  # dict
    if not disk_name:
        raise exceptions.TestError("Disk name must be provided for backup disk xml.")
    backup_disk_xml.name = disk_name
    if disk_type:
        backup_disk_xml.type = disk_type
    if enable_backup:
        backup_disk_xml.backup = enable_backup
    if exportname:
        backup_disk_xml.exportname = exportname
    if exportbitmap:
        backup_disk_xml.exportbitmap = exportbitmap
    if backupmode:
        backup_disk_xml.backupmode = backupmode
    if incremental:
        backup_disk_xml.incremental = incremental
    if backup_target:
        if not isinstance(backup_target, dict):
            raise exceptions.TestError("disk target tag should be defined by a dict.")
        backup_disk_xml.target = backup_target
    if backup_driver:
        if not isinstance(backup_driver, dict):
            raise exceptions.TestError("disk driver tag should be defined by a dict.")
        backup_disk_xml.driver = backup_driver
    if backup_scratch:
        if not isinstance(backup_scratch, dict):
            raise exceptions.TestError("disk scratch tag should be defined by a dict.")
        backup_disk_xml.scratch = backup_scratch
    backup_disk_xml.xmltreefile.write()

    utils_misc.wait_for(lambda: os.path.exists(backup_disk_xml.xml), 5)
    return backup_disk_xml


def pull_incremental_backup_to_file(nbd_params, target_path,
                                    bitmap_name, file_size):
    """
    Dump pull-mode incremental backup data to a file

    :param nbd_params: The params of nbd service which provide the backup data
    :param target_path: The path of the file to dump the data
    :param bitmap_name: The dirty bitmap used for the backup
    :param file_size: The size of the target file to be preapred
    """
    nbd_protocol = nbd_params.get("nbd_protocol", "tcp")
    nbd_export = nbd_params.get("nbd_export", "vdb")
    tls_dir = nbd_params.get("tls_dir")
    cmd = "qemu-img create -f qcow2 %s %s" % (target_path, file_size)
    logging.debug(process.run(cmd, shell=True).stdout_text)
    map_from = "--image-opts driver=nbd,export=%s,server.type=%s"
    map_from += ",%s"
    map_from += ",x-dirty-bitmap=qemu:dirty-bitmap:%s"
    if nbd_protocol == "tcp":
        nbd_hostname = nbd_params.get("nbd_hostname", "127.0.0.1")
        nbd_tcp_port = nbd_params.get("nbd_tcp_port", "10809")
        nbd_type = "inet"
        nbd_server = "server.host=%s,server.port=%s" % (nbd_hostname, nbd_tcp_port)
        nbd_image_path = "nbd://%s:%s/%s" % (nbd_hostname, nbd_tcp_port, nbd_export)
        if tls_dir:
            qemu_rebase_cmd = """qemu-img rebase -u -f qcow2 -F raw \
                    -b 'json:{"file":{"driver":"nbd", \
                    "server":{"host":"'%s'", \
                    "port":'%s', "type":"inet"}, \
                    "export":"'%s'", \
                    "tls-creds":"tls0"}}' %s"""
            qemu_rebase_cmd %= (nbd_hostname, nbd_tcp_port, nbd_export, target_path)
            map_from += ",tls-creds=tls0"
            map_from %= (nbd_export, nbd_type, nbd_server, bitmap_name)
            qemu_map_cmd = "qemu-img map --object "\
                           "tls-creds-x509,id=tls0,endpoint=client,dir=%s "\
                           "--output=json %s -U" % (tls_dir, map_from)
        else:
            qemu_rebase_cmd = "qemu-img rebase -u -f qcow2 -F raw -b %s %s"
            qemu_rebase_cmd %= (nbd_image_path, target_path)
            map_from %= (nbd_export, nbd_type, nbd_server, bitmap_name)
            qemu_map_cmd = "qemu-img map --output=json %s -U" % map_from
    elif nbd_protocol == "unix":
        nbd_type = "unix"
        nbd_socket = nbd_params.get("nbd_socket", "/tmp/pull_backup.socket")
        nbd_server = "server.path=%s" % nbd_socket
        nbd_image_path = "nbd+unix:///%s?socket=%s" % (nbd_export, nbd_socket)
        qemu_rebase_cmd = "qemu-img rebase -u -f qcow2 -F raw -b %s %s"
        qemu_rebase_cmd %= (nbd_image_path, target_path)
        map_from %= (nbd_export, nbd_type, nbd_server, bitmap_name)
        qemu_map_cmd = "qemu-img map --output=json %s -U" % map_from
    # Rebase backup file to nbd image
    process.run(qemu_rebase_cmd, shell=True)
    # Get the nbd export's data map
    result = process.run(qemu_map_cmd, shell=True).stdout_text
    json_data = json.loads(result)
    # Dump backup data to target image
    for entry in json_data:
        if not entry["data"]:
            qemu_io_cmd = "qemu-io -C -c \"r %s %s\" -f qcow2 %s"
            qemu_io_cmd %= (entry["start"], entry["length"], target_path)
            if tls_dir:
                qemu_io_cmd += " --object tls-creds-x509,id=tls0,"\
                               "endpoint=client,dir=%s" % tls_dir
            process.run(qemu_io_cmd, shell=True)
    # remove the backing file info for the target image
    qemu_rebase_cmd = "qemu-img rebase -u -f qcow2 -b '' %s" % target_path
    process.run(qemu_rebase_cmd, shell=True)


def pull_full_backup_to_file(nbd_params, target_path):
    """
    Dump pull-mode full backup data to a file

    :param nbd_export: The nbd export info of the backup job
    :param target_path: The path of the file to dump the data
    """
    protocol = nbd_params.get("nbd_protocol")
    hostname = nbd_params.get("nbd_hostname")
    port = nbd_params.get("nbd_tcp_port")
    export = nbd_params.get("nbd_export")
    tls_dir = nbd_params.get("tls_dir")
    socket = nbd_params.get("nbd_socket")
    if protocol == "tcp":
        if not tls_dir:
            cmd = "qemu-img convert -f raw nbd://%s:%s/%s " \
                  "-O qcow2 %s" % (hostname, port, export, target_path)
        else:
            json_input = ("'json:{\"file\":{\"driver\":\"nbd\","
                          "\"server\":{\"host\":\"%s\",\"port\":%s,"
                          "\"type\":\"inet\"},\"export\":\"%s\","
                          "\"tls-creds\":\"tls0\"}}'") % (hostname, port, export)
            cmd = ("qemu-img convert -O qcow2 --object tls-creds-x509,id=tls0,"
                   "endpoint=client,dir=%s %s %s") % (tls_dir, json_input, target_path)
    elif protocol == "unix":
        cmd = "qemu-img convert -f raw nbd+unix:///%s?socket=%s " \
              "-O qcow2 %s" % (export, socket, target_path)
    process.run(cmd, shell=True)


def get_img_data_size(img_path, driver='qcow2'):
    """
    Get the actual data size of a image

    :param img_path: The path to the image
    :return: The actual data size of the image in 'MB'.
    """
    cmd = "qemu-img map -f %s %s --output=json -U" % (driver, img_path)
    result = process.run(cmd, shell=True).stdout_text
    json_data = json.loads(result)
    summary = 0
    for entry in json_data:
        if entry["data"]:
            summary += entry["length"]
    return summary/1024/1024


def get_img_data_map(img_path, driver='qcow2'):
    """
    Get the data map of a image

    :prama img_path: The path to the image
    :return: The data map of the image
    """
    cmd = "qemu-img map -f %s %s --output=json -U" % (driver, img_path)
    result = process.run(cmd, shell=True).stdout_text
    json_data = json.loads(result)
    data_map = []
    for entry in json_data:
        if entry["data"]:
            data_map.append(entry)
    return data_map


def dump_data_to_file(data_map, source_file_path, target_file_path, driver='qcow2'):
    """
    Dump real data from the source file to a target file according to data map

    :param data_map: The data map used to indicate which part of the image
    containing real data
    :param source_file_path: The path to the source file
    :param target_file_path: The path to the target file
    :param driver: The image format of the source file
    """
    with open(target_file_path, 'w'):
        cmd = "qemu-io -f %s -c \"read -vC %s %s\" %s -r -U  | sed '$d'"
        for entry in data_map:
            cmd %= (driver, entry["start"], entry["length"], source_file_path)
            cmd += ">> %s" % target_file_path
            process.run(cmd, shell=True)


def cmp_backup_data(original_file, backup_file,
                    original_file_driver='qcow2', backup_file_driver='qcow2'):
    """
    Compare the backuped data and the original data

    :param original_file: The file containing the original data
    :param backup_file: The file containing the backuped data
    :param original_file_driver: Original image format
    :param backup_file_driver: Backup image format
    :return: Trun if data is correct, false if not
    """
    original_file_dump = original_file + ".dump"
    backup_file_dump = backup_file + ".dump"
    data_map = get_img_data_map(backup_file, backup_file_driver)
    dump_data_to_file(data_map, original_file, original_file_dump, original_file_driver)
    dump_data_to_file(data_map, backup_file, backup_file_dump, backup_file_driver)
    if filecmp.cmp(original_file_dump, backup_file_dump):
        return True
